--[[
	Target: In this example script, it is shown how P2 Engine can reproduce all the data for the three project examples of the book written by Mario Vanhoucke, "Integrated Project Management and Control: First Comes the Theory, then the Practice" 
	More information: www.p2engine.com
	Start with: main_procedure_controlling_three_projects
	Last update: May 4, 2015
	Input files: This script requires input files that must be placed in a folder MY_DATA. These files are entitled project_1.rcp, project_2.rcp and project_3.rcp 
	Arguments: This script requires one argument, i.e. the ID of the input file (i.e. 1, 2 or 3 for files project_1.rcp, project_2.rcp or project_3.rcp, respectively)
	Note: Note that the output results can deviate from the analysis and metric values in the book due to seed values or small differences in implementation
--]]

-- Define your path where you have placed your input project_x.rcp files and where you want to see your output project_x.out files (MUST BE CHANGED BY THE USER!!)
MY_DATA=string.format('/Users/mariovanhoucke/Documenten van Mario/Research Folder/Academic Topics/Boeken/Finished/IPMC (Springer)/pictures/projecten/') 

-- Call this script using the terminal in mac with the following command:
-- "/Users/mariovanhoucke/Documenten van Mario/XCode Projecten/OR-AS research/research/p2engine/build/Debug/p2" "/Users/mariovanhoucke/Documenten van Mario/Lua Scripts/tutorial/IPMCExamples/UserCode (recommended)/example_script_IPMC_book.lua" 1 (or 2 or 3 for project 2 or 3)
-- A similar call can be done in Windows

function create_project_object() 
	MyProject=P2Project:new()
end

function create_schedule_object() 
	MySchedule=P2Schedule:new(MyProject)
end

function create_tracking_object()
	MyTracking=(MyTracking or P2Tracking:new(MySchedule))
end

function create_simulator_object()
	MySimulator=P2Simulator:new(MySchedule)
end

function delete_objects() 
	MySimulator=nil
	MyProject=nil 
	MySchedule=nil 
	MyTracking=nil
end

function read_my_patterson_file(filename) 
	MyReader=P2Reader(MyProject,MySchedule) 
	result=MyReader:read_patterson_file(filename) 
	if(result==0) then
		io.write("\nERROR: No input file with name ",filename," found! \n")
	end
	MyReader=nil
end

function add_costs(projectID)
	-- Assign costs per time unit to activities (cost per unit = total cost / activity duration)
	if projectID==1 then
		cost_per_time_unit={1000,7500,7500,1500,1300,1300,900,1000,1500,1500,3000,1000,1000,1000,1000,2000,5000}
	elseif projectID==2 then
		cost_per_time_unit={5000,2500,700,1500,5000,1500,5000,6500,6000,500,1100,1900,600,9000,1500,100,5000}
	else
		cost_per_time_unit={10000,10000,30000,10000,20000,25000,10000,15000,25000,5000,4000,10000,10000,15000,3000,10000,20000}		
	end

	for act,cost in ipairs(cost_per_time_unit) do
		io.write("\nAssign cost ",cost," to ",act)
		MySchedule:set_variable_cost(act,cost)
	end
end


function measure_network_topology() 
	MyGenerator=P2Generator:new(MyProject,MySchedule) 
	SP,AD,LA,TF=MyGenerator:calculate_topological_structure() 
	MyGenerator=nil
end

function construct_baseline_schedule() 
	MySchedule:earliest_start_schedule_with_float_factor(0) 
	PD=MySchedule:get_project_pd()
	BAC=MySchedule:get_project_bac()
end

function set_progress_data(projectID) 
	if projectID==1 then
       RealStart={0,1,1,4,4,4,3,3,4,4,9,7,5,5,7,12,12}
       RealDuration={1,3,3,3,5,3,4,2,1,1,3,1,2,1,3,2,1}
    elseif projectID==2 then
       RealStart={0,1,3,5,7,7,8,9,12,13,12,12,10,17,19,20,22}
       RealDuration={1,2,4,1,4,2,3,1,2,5,4,3,5,5,3,5,2}
    else
       RealStart={0,2,5,8,10,13,14,14,14,16,17,19,17,19,21,22,23}
       RealDuration={2,3,5,2,6,1,2,1,1,1,2,2,2,1,2,1,1}
    end
    RealCost={}
	nract=MySchedule:get_number_of_activities() 
	for act=1,nract do
		RealCost[act]=MySchedule:get_variable_cost(act)*RealDuration[act] 
	end
end

function simulate_progress_manually(RealStart,RealDuration) 
	MyTracking:reset_tracking_object()
	-- [Construct period tracking periods]
	local timePtr=1
    local projectEnded=false
    while projectEnded==false do
		nrFinish=0
		periodID=MyTracking:add_tracking_period(timePtr) 
		nract=MySchedule:get_number_of_activities()
		for act=1,nract do
			if timePtr<RealStart[act] then 
				--[Activity: not started yet] 
				PC=0
				AD=0
				AS=99999999
			elseif timePtr >= RealStart[act]+RealDuration[act] then
				--[Activity is finished] 
				PC=1 
				AD=RealDuration[act] 
				AS=RealStart[act] 
				nrFinish=nrFinish+1
			else
				--[Activity is busy] 
				PC=(timePtr-RealStart[act])/RealDuration[act] 
				AD=timePtr-RealStart[act]
				AS=RealStart[act]
		end
		MyTracking:set_actual_start(act,AS,periodID) 
		MyTracking:set_percentage_completed(act,PC,periodID) 
		MyTracking:set_actual_duration(act,AD,periodID) 
		AC=MyTracking:get_planned_actual_cost(act,periodID)
		MyTracking:set_actual_cost(act,AC,periodID) end
		-- [Construct an EVM report for this tracking period] 
		MyTracking:construct_evm_report(periodID) 
		timePtr=timePtr+1
		if nrFinish==nract then projectEnded=true end
	end 
end

function performance_measurement()
	nract=MySchedule:get_number_of_activities() 
	nrtrackper=MyTracking:get_number_of_tracking_periods() 
	RC=MyTracking:get_real_project_cost() 
	RD=MyTracking:get_real_project_duration()
	-- [The EVM key metrics]
	for period=1,nrtrackper do
		PV=MyTracking:get_project_pv(period) 
		EV=MyTracking:get_project_ev(period) 
		AC=MyTracking:get_project_ac(period) 
		ES=MyTracking:get_project_es(period)
	end
	-- [The EVM performance measures] 
	for period=1,nrtrackper do
		SV=MyTracking:get_project_sv(period) 
		CV=MyTracking:get_project_cv(period) 
		SPI=MyTracking:get_project_spi(period) 
		CPI=MyTracking:get_project_cpi(period) 
		SVT=MyTracking:get_project_svt(period) 
		SPIT=MyTracking:get_project_spit(period)
		p_factor=MyTracking:get_project_p_factor(period)
	end 
end


function forecasting()
	for period=1,nrtrackper do
		PF1,PFCPI,PFSPI,PFSPIT,PFSCI,PFSCIT,PFwSPICPI,PFwSPITCPI=MyTracking:get_eac_cost(period)
		PV1,PV2,PV3,ED1,ED2,ED3,ES1,ES2,ES3=MyTracking:get_eac_time(period) 
	end
end

function run_a_schedule_risk_analysis(nrruns) 
	MySimulator:reset_simulation_object(MySchedule) 
	MySimulator:create_monte_carlo_object()
	-- [Increase precision]
	nract=MySchedule:get_number_of_activities() 
	for act=1,nract do
		dur=MySchedule:get_duration(act) 
		MySchedule:change_duration(act,dur*5)
	end
	-- [Assign triangular distributions to the project activities] 
	prob={0,100,0}
	lower=20
	upper=20
	for act=1,MySchedule:get_number_of_activities() do
		MySimulator:set_number_advanced_distributions(act,3) 
		a=math.floor((1-(lower/100))*MySchedule:get_duration(act)) 
		m=math.floor(MySchedule:get_duration(act)) 
		b=math.ceil((1+(upper/100))*MySchedule:get_duration(act)) 
		val={a,m,b}
		for x=1,3 do
			MySimulator:set_advanced_distributions(act,x,val[x],prob[x])
		end 
	end
	time_unit=5
	MySimulator:set_periodic_time_unit(time_unit) 
	MySimulator:set_callback("sim_callback") 
	MySimulator:run_simulation(nrruns,11,1,1,0,0)
end


function get_sensitivity_measures()
	for act=1,MySchedule:get_number_of_activities() do
		CI,SI,SSI,r,rho,tau=MySimulator:get_activity_time_sensitivity(act)
		c_r,c_rho,c_tau=MySimulator:get_activity_cost_sensitivity(act) 
	end
end

function label_activities(projectID)
	ActName={}
	if projectID==1 then
		ActName[1]='Kick-off meeting                            '
		ActName[2]='Define database specifications              '
		ActName[3]='Define user interface specifications        '
		ActName[4]='Develop maintenance cost database           '
		ActName[5]='Develop improvement cost database           '
		ActName[6]='Develop production time database            '
		ActName[7]='Develop IT department user interface        '
		ActName[8]='Develop accounting department user interface'
		ActName[9]='Develop management user interface           '
		ActName[10]='Develop factory floor user interface       '
		ActName[11]='Database integration                       '
		ActName[12]='End user feedback IT department            '
		ActName[13]='End user feedback accounting department    '
		ActName[14]='End user feedback management               '
		ActName[15]='End user feedback factory floor            '
		ActName[16]='Full product integration                   '
		ActName[17]='Project evaluation                         '
	elseif projectID==2 then	
		ActName[1]='Staff needs survey                          '
		ActName[2]='ICT committee meeting                       '
		ActName[3]='Requirements study                          '
		ActName[4]='Request transport system supplier proposals '
		ActName[5]='Negotiate best transport system offer       '
		ActName[6]='Request database supplier proposals         '
		ActName[7]='Negotiate best database offer               '
		ActName[8]='Kick-off meeting with staff                 '
		ActName[9]='Installation of transport system            '
		ActName[10]='Installation of server                     '
		ActName[11]='PC training for nursing staff              '
		ActName[12]='Code list creation                         '
		ActName[13]='Filling the database                       '
		ActName[14]='Practical training for nursing staff       '
		ActName[15]='Final check                                '
		ActName[16]='Live pilot period                          '
		ActName[17]='Project evaluation                         '
	else
		ActName[1]='Demolition phase 1: roof & utilities'
		ActName[2]='Demolition phase 2: masonry & floor '
		ActName[3]='Structural work & sewage            '
		ActName[4]='Facade renovation                   '
		ActName[5]='Roofing                             '
		ActName[6]='Exterior joinery                    '
		ActName[7]='Electricity                         '
		ActName[8]='Plumbing                            '
		ActName[9]='Heating & ventilation               '
		ActName[10]='Floor insulation & screed          '
		ActName[11]='Plasterboard                       '
		ActName[12]='Devices & connections              '
		ActName[13]='Suspended ceiling                  '
		ActName[14]='Interior joinery                   '
		ActName[15]='Painting                           '
		ActName[16]='Linoleum flooring                  '
		ActName[17]='Furniture & cleanup                '
	end
end

function write_baseline_schedule_information()
	io.write('\n\nWrite baseline schedule information')
	nract=MySchedule:get_number_of_activities()
	PD=MySchedule:get_project_pd()
	BAC=MySchedule:get_project_bac()
	measure_network_topology()
	io.write('\nNumber of activities ',nract)
	SP = SP / 100
	AD = AD / 100
	LA = LA / 100
	TF = TF / 100
	io.write('\n\tSerial/Parallel indicator ',SP)
	io.write('\n\tActivity distribution ',AD)
	io.write('\n\tLength of Arcs ',LA)
	io.write('\n\tTopological Float ',TF)
	io.write('\nPlanned Duration ',PD)
	io.write('\nBudget At Completion ',BAC)
	io.write('\nActivity Start Times:')
	local PlannedStartTimes={}
	for act=1,nract do
		PlannedStartTimes[act]=MySchedule:get_start(act)
		io.write('\n\tActivity ',act,' starts at ',MySchedule:get_start(act))
		critical=MySchedule:get_critical_activity(act)
		if (critical==0) then
			io.write(' and is NOT critical')
		else
			io.write(' and is critical')
		end
	--	io.write('\t',ActName[act])
	end
	io.write('\n')
end

function write_baseline_and_progress_schedule()
	nract=MySchedule:get_number_of_activities()
	local RD=0
	local RAC=0
	for act=1,nract do
		start=MySchedule:get_start(act)
		duration=MySchedule:get_duration(act)
		io.write('\n',act,'\t',start,'\t',duration)
		io.write('\t',start+duration,'\t',MySchedule:get_critical_activity(act))
		io.write('\t',MySchedule:get_variable_cost(act))
	end
	io.write('\n\n')
	for act=1,nract do
		start=RealStart[act]
		duration=RealDuration[act]
		cost=RealCost[act]
		RD=math.max(RD,start+duration)
		RAC=RAC+cost
		io.write('\n',act,'\t',start,'\t',duration)
		io.write('\t',RealStart[act]+RealDuration[act],'\t',MySchedule:get_critical_activity(act))
		io.write('\t',MySchedule:get_variable_cost(act))
	end
	
	-- idem maar in een ander formaat
--	for act=1,nract do
--		start=MySchedule:get_start(act)
--		duration=MySchedule:get_duration(act)
--		io.write('\n',act,'\t',start,'\t',duration)
--		io.write('\t',start+duration,'\t',MySchedule:get_critical_activity(act))
--		io.write('\t',MySchedule:get_variable_cost(act))
--		io.write('\n',act,'\t',RealStart[act],'\t',RealDuration[act])
--		io.write('\t',RealStart[act]+RealDuration[act],'\t',MySchedule:get_critical_activity(act))
--		io.write('\t',MySchedule:get_variable_cost(act))
--	end
	io.write('\nReal Duration ',RD)
	io.write('\nReal Cost ',RAC)
end


function main_procedure_controlling_three_projects()
	-- Create the three objects
	create_project_object()
	create_schedule_object()
	create_tracking_object()

	-- Read the project ID (1, 2 or 3) as input	
	local projectID = tonumber(arg[1])
	filename = 'project_'..arg[1]..'.rcp'
	io.write("\n You are now reading file ",filename, "! ")

	-- Read the project networks (project_1.rcp or project_2.rcp or project_3.rcp)
	read_my_patterson_file(MY_DATA..filename)
	label_activities(projectID)
	add_costs(projectID)

	-- Write baseline schedule
	write_baseline_schedule_information()
	
	-- Imitate project progress (manually)
	set_progress_data(projectID) 
	simulate_progress_manually(RealStart,RealDuration) 

	-- Write progress information
	write_baseline_and_progress_schedule()  -- call after "imitate_progress"

	-- Create an EVM data table (similar to table 2.2 of the book "Measuring Time")
	create_EVM_table2_2()
	
	-- Schedule risk analysis
	create_simulator_object()
		
	MySchedule:put_batch_calculator_on()
	nract=MySchedule:get_number_of_activities()
	for act=1,nract do
		dur=MySchedule:get_duration(act)
		MySchedule:change_duration(act,dur*5)
	end
	MySchedule:put_batch_calculator_off()
	
	nrruns=20
	run_a_schedule_risk_analysis(nrruns)

	print_SRA_output_table()
	
	-- Delete all objects
	delete_objects()
	collectgarbage()
end

function sim_callback(p1,p2,p3)
--[[
	Procedure set in "set_callback"
	This procedure is called at every run (1 to nrruns) of each simulation and is a powerful tool
	to interrupt simulations based on conditions or to write data to the screen or a file for every simulation.

	For more information:
	visit http://www.p2engine.com/p2simulator/set_callback 
	or look at other example scripts available at http://www.p2engine.com/examples
	
	In this example, the tracking data is generated in a tracking object for each run. 
	All the data of this tracking object can then be written to a text file and can be analysed. 
	This is exactly how PhD students at OR&S work (see www.projectmanagement.ugent.be).
	In the current example, the following information is written to the screen (but never used since it was only for illustrative purposes)
		- The number of tracking periods
		- The real (simulated) duration of the project 
		- The real (simulated) cost of the project
		- The real (simulated) duration of each activity
		- Project is early, on time, late
	Obviously, a lot of other data can be generated. 
	Imagine what databases you can create to analyse the data and do research!
--]]
	do return 0 end --do not write all data to screen!
	io.write('\nSimulation run ',p1,'   ')
	MyTracking:reset_tracking_object()
	MySimulator:write_output_to_tracking_list(MyTracking)
		
	nrperiods=MyTracking:get_number_of_tracking_periods()
	io.write('\t'..nrperiods)	
	RD=MyTracking:get_real_project_duration()
	RC=MyTracking:get_real_project_cost()
	io.write("\t",RD)
	io.write("\t",RC)
	for act=1,MySchedule:get_number_of_activities() do
		realdur=MyTracking:get_actual_duration(act,nrperiods)
		io.write(string.format('\t%d',realdur))
	end
	PD=MySchedule:get_project_pd()
	if PD<RD then io.write('\tlate')
		elseif PD>RD then io.write('\tearly') 
		else io.write('\ton time')
	end
	return 0
end


function print_SRA_output_table()	
	io.write('\nTime sensitivity table')
	io.write('\nActivity ID\tCI\tSI\tSSI\tCRI(r)\tCRI(rho)\tCRI(tau)')
	for act=1,MySchedule:get_number_of_activities() do
		sensitivity_matrix={MySimulator:get_activity_time_sensitivity(act)}
		io.write('\n',act)
		for i=1,6 do
			io.write(string.format('\t%8.2f',sensitivity_matrix[i]))
		end
	end	
	io.write('\nCost sensitivity table')
	io.write('\nActivity ID\t  CRI(r)\tCRI(rho)\tCRI(tau)')
	for act=1,MySchedule:get_number_of_activities() do
		sensitivity_matrix={MySimulator:get_activity_cost_sensitivity(act)}
		io.write('\n',act)
		for i=1,3 do
			io.write(string.format('\t%8.2f',sensitivity_matrix[i]))
		end
	end	
end	


function create_EVM_table2_2()
	filename = 'project_'..arg[1]..'.out'
	io.write('\nAll EVM calculations will be printed in a text file with the name ',filename)
	io.write('\nThe structure of this txt file will be similar to table 2.2 of Measuring Time\n')
	-- Create a table with the EVM key metrics and performance measures (including p-factor) and write to file
	-- (see table 2.2 of the book "Measuring Time")

	outfilename = 'project_'..arg[1]..'.out'

	-- Create an empty file
	output_file=string.format(MY_DATA..outfilename)
	file=io.open(output_file,"w")

	-- Get the number of activities, the planned project duration and the number of tracking periods (which is equal to the real duration since each tracking period increases with 1 time unit in this example)
	nract=MySchedule:get_number_of_activities()					-- 17 in this example
	nrbaselineper=MySchedule:get_project_pd()					
	nrtrackper=MyTracking:get_number_of_tracking_periods()		-- Hoeveel? Per week?

	-- Write header
	file:write('Day')
	for period=1,math.max(nrtrackper,nrbaselineper) do
		file:write('\t',period)
	end

	file:write('\n\t\t\t\t\t\t\t\tTraditional Earned Value Calculations')

	-- Write the Earned Value (EV)
	file:write('\nEV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_ev(period)))
	end

	-- Write the Actual Cost (AC)
	file:write('\nAC\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_ac(period),'\t')
	end

	-- Write the Planned Value (PV)
	file:write('\nPV\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_pv(period),'\t')
	end
	
	--[[ In case the project finishes early, no tracking periods have been created 
	   between the finish and the PD and hence the PV curve is not known in this interval.
	   Therefore, new fictitious tracking periods are generated and only the PV is calculated --]]
	if nrtrackper<nrbaselineper then
		local timePtr=nrtrackper+1
		while timePtr<=nrbaselineper do
			periodID=MyTracking:add_tracking_period(timePtr)
			MyTracking:construct_evm_report(periodID)
			file:write(MyTracking:get_project_pv(periodID),'\t')
			timePtr=timePtr+1
		end	
	end

	file:write('\n')

	-- Write the EVM performance measures
	file:write('\nSV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_sv(period)))
	end
	file:write('\nCV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.0f\t',MyTracking:get_project_cv(period)))
	end	
	file:write('\nSPI\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spi(period)))
	end
	file:write('\nCPI\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_cpi(period)))
	end
	
	file:write('\n')
	file:write('\nES\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_es(period)))
	end
	file:write('\nSV(t)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_svt(period)))
	end
	file:write('\nSPI(t)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spit(period)))
	end
	
	file:write('\n\t\t\t\t\t\t\t\tEffective Earned Value Calculations')
	file:write('\np-factor\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_p_factor(period)))
		--io.write(string.format('%2.2f\t',MyTracking:get_project_p_factor(period)))
	end
	
	-- Effective EVM is not incorporated in P2 Engine but can be easily calculated as illustrated along the following lines
	-- Note: Try to avoid to program procedures yourself when available in P2 Engine

	-- Set the estimated portion of EV(r) that is usable and required no rework
	RPercent=0.9
		
	-- Calculate and write the effective Earned Value (EV(e))
	ev_e={}
	file:write('\nEV(e)\t')
	for period=1,nrtrackper do
		ev=MyTracking:get_project_ev(period)
		p=MyTracking:get_project_p_factor(period)
		ev_e[period]=ev*p+RPercent*(1-p)*ev
		file:write(string.format('%2.2f\t',ev_e[period]))
	end

	-- Write the EVM performance measures
	file:write('\nSV(e)\t')
	for period=1,nrtrackper do
		sv_e=ev_e[period]-MyTracking:get_project_pv(period)
		file:write(string.format('%2.2f\t',sv_e))
	end
	file:write('\nCV(e)\t')
	for period=1,nrtrackper do
		cv_e=ev_e[period]-MyTracking:get_project_ac(period)
		file:write(string.format('%2.2f\t',cv_e))
	end	
	file:write('\nSPI(e)\t')
	for period=1,nrtrackper do
		spi_e=ev_e[period]/MyTracking:get_project_pv(period)
		file:write(string.format('%2.2f\t',spi_e))
	end
	file:write('\nCPI(e)\t')
	for period=1,nrtrackper do
		cpi_e=ev_e[period]/MyTracking:get_project_ac(period)
		file:write(string.format('%2.2f\t',cpi_e))
	end

	-- Calculate and write the effective Earned Schedule (ES(e))
	es_e={}
	for period=1,nrtrackper do
		timePtr=0
		for t=1,nrtrackper do
			pvt=MyTracking:get_project_pv(t)
			if pvt>=ev_e[period] then 
				timePtr=t-1
				break
			end
		end	
		if timePtr==0 then
			pvt=0
		else
			pvt=MyTracking:get_project_pv(timePtr)
		end	
		pvtplus=MyTracking:get_project_pv(timePtr+1)
		es_e[period]=timePtr+(ev_e[period]-pvt)/(pvtplus-pvt)
	end

	file:write('\n\nES(e)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',es_e[period]))
	end

	file:write('\nSV(t)(e)\t')
	for period=1,nrtrackper do
		sv_e=es_e[period]-MyTracking:get_status_date(period)
		file:write(string.format('%2.2f\t',sv_e))
	end

	file:write('\nSPI(t)(e)\t')
	for period=1,nrtrackper do
		spi_e=es_e[period]/MyTracking:get_status_date(period)
		file:write(string.format('%2.2f\t',spi_e))
	end
	
	file:write('\n\nCost Forecasting (EAC)')
	file:write('\nReal Cost\t',MyTracking:get_real_project_cost())
	file:write('\nPF=1\tPF=CPI\tPF=SPI\tPF=SPI(t)\tPF=SCI\tPF=SCI(t)\tPF=w(SPI,CPI)\tPF=w(SPI(t),CPI)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\n%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f',EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI))
	end

	file:write('\n\n\n\nTime Forecasting (EAC(t))')
	file:write('\nReal Time\t',MyTracking:get_real_project_duration())
	file:write('\nPV1\tPV2\tPV3\tED1\tED2\tED3\tES1\tES2\tES3')
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\n%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f\t%2.2f',EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3))
	end

	-- write to screen (in other format)
	file:write('\n\nCOST FORECASTING')	
	file:write('\nPF=1')	
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PF1))--,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI))
	end

	file:write('\nPF=CPI')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFCPI))
	end

	file:write('\nPF=SPI')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSPI))
	end

	file:write('\nPF=SPI(t)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSPIT))
	end
	
	file:write('\nPF=SCI')	
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSCI))
	end

	file:write('\nPF=SCI(t)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFSCIT))
	end

	file:write('\nPF=w(SPI,CPI)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFweightedSPICPI))
	end

	file:write('\nPF=w(SPI(t),CPI)')
	for period=1,nrtrackper do
		EAC_PF1,EAC_PFCPI,EAC_PFSPI,EAC_PFSPIT,EAC_PFSCI,EAC_PFSCIT,EAC_PFweightedSPICPI,EAC_PFweightedSPITCPI=MyTracking:get_eac_cost(period)
		file:write(string.format('\t%2.2f',EAC_PFweightedSPITCPI))
	end
	
	file:write('\n\nTIME FORECASTING')	
	file:write('\nPV-PF=1')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_PV1))
	end

	file:write('\nPV-PF=SPI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_PV2))
	end

	file:write('\nPV-PF=SCI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_PV3))
	end

	file:write('\nED-PF=1')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ED1))
	end

	file:write('\nED-PF=SPI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ED2))
	end

	file:write('\nED-PF=SCI')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ED3))
	end

	file:write('\nES-PF=1')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ES1))
	end

	file:write('\nES-PF=SPI(t)')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ES2))
	end

	file:write('\nES-PF=SCI(t)')	
	for period=1,nrtrackper do
		EACt_PV1,EACt_PV2,EACt_PV3,EACt_ED1,EACt_ED2,EACt_ED3,EACt_ES1, EACt_ES2,EACt_ES3=MyTracking:get_eac_time(period)
		file:write(string.format('\t%2.2f',EACt_ES3))
	end

	-- Close the file
	file:flush()
	file:close()
end

main_procedure_controlling_three_projects()
