--[[
	Target: In this example script, it is shown how the project example published in the book 
			"Measuring Time - Improving Project Performance using Earned Value Management" 
			(Vanhoucke, pages 18-19, figures 1.7 and 1.8 and tables 1.5 and 1.6)
			can be reproduced using P2 Engine. 
	More information: www.p2engine.com
	Main procedure: main_procedure_chapter1
	Uses the following subprocedure(s): "subprocedure_create_objects" and "subprocedure_manual_simulation"
	Classes: P2Project, P2Schedule and P2Tracking
	Last update: August 27, 2012
--]]

-- Define your path where you unzip your downloaded file from www.p2engine.com (MUST BE CHANGED BY THE USER!!)
MY_PATH=string.format('/Users/mariovanhoucke/Documenten van Mario/Lua Scripts/tutorial/')

-- Other paths are defined automatically (no need to change these paths!)
MY_SCRIPTS=MY_PATH..'MeasuringTimeExamples/'
MY_GENERAL=MY_PATH..'general/'
dofile(MY_SCRIPTS..'subprocedure_manual_simulation.lua')
dofile(MY_GENERAL..'subprocedure_create_objects.lua')

function main_procedure_chapter1()
	-- Create the three objects
	create_project_object()
	create_schedule_object()
	create_tracking_object()

	-- Add project and schedule data
	add_activities()
	add_costs()
	add_relations()
	write_project_data_to_screen()

	-- Imitate project progress (manually)
	local RealStartTimes={0,0,0,6,1,4,6,10,11,6}
	local RealDurations={6,11,1,4,3,2,8,8,5,4}
	simulate_progress_manually(RealStartTimes,RealDurations)

	-- Create EVM data (tables 1.5 and 1.6 of the book "Measuring Time")
	create_forecasting_accuracy_table1_5()	-- table 1.5
	create_EVM_table1_6()					-- table 1.6

	delete_tracking_object()
	delete_schedule_object()
	delete_project_object()
	release_all_memory()
end

function add_activities()
	-- Put the batch calculator to "ON"
	-- P2 Engine will now only accept data, and no automatic scheduling procedures are called
	MySchedule:put_batch_calculator_on()
	
	-- Add activities to the project and the schedule
	MySchedule:add_activity_to_schedule(4)		-- this function returns the activity ID, starting with one and increasing in steps of 1
	MySchedule:add_activity_to_schedule(9)
	MySchedule:add_activity_to_schedule(1)
	MySchedule:add_activity_to_schedule(4)
	MySchedule:add_activity_to_schedule(5)
	MySchedule:add_activity_to_schedule(1)
	MySchedule:add_activity_to_schedule(7)
	MySchedule:add_activity_to_schedule(8)
	MySchedule:add_activity_to_schedule(3)
	MySchedule:add_activity_to_schedule(3)
end

function add_costs()
	-- Assign costs per time unit to activities (cost per unit = total cost / activity duration)
	-- In the example, the costs per unit are assigned in a for loop using an array of costs
	cost_per_time_unit = {20, 4, 25, 20, 4, 120, 3, 1, 2, 20}
	for act,cost in ipairs(cost_per_time_unit) do
		MySchedule:set_variable_cost(act,cost)
	end
end

function add_relations()
	-- Add the precedence relations to the project network (all relations are Finish-Start relation with zero time lag)
	MySchedule:add_relation(1,4,3,0)
	MySchedule:add_relation(4,8,3,0)
	MySchedule:add_relation(2,9,3,0)
	MySchedule:add_relation(3,5,3,0)
	MySchedule:add_relation(5,6,3,0)
	MySchedule:add_relation(6,7,3,0)
	MySchedule:add_relation(6,10,3,0)

	-- All data has been added: Put the batch calculator to "OFF"
	-- P2 Engine will now automatically schedule the activities as an earliest start schedule
	MySchedule:put_batch_calculator_off()
end

function write_project_data_to_screen()

	-- Write the number of activities and their durations to the screen
	io.write('\nNumber of activities added to the project = ',MySchedule:get_number_of_activities())
	for act=1,MySchedule:get_number_of_activities() do
		io.write('\n\tDuration of activity ',act,' = ',MySchedule:get_duration(act))
	end

	-- Write the variable cost per unit and total cost to the screen
	for act = 1,10 do
		io.write("\nActivity ".. act..": \n\t The variable cost: "..MySchedule:get_variable_cost(act))
		io.write("\n\t The total cost: "..MySchedule:get_total_activity_cost(act))	
	end

	-- Write the Planned Duration (PD) of the project (see figure 1.8 of the book "Measuring Time")
	io.write('\nThe planned duration of the baseline schedule is ',MySchedule:get_project_pd())

	-- Write the Budget At Completion to the screen
	io.write("\nThe project has a planned budget at completion of "..MySchedule:get_project_bac())
	
	-- Write the Earliest Start Schedule (ESS) (see figure 1.8 of the book "Measuring Time")
	for act = 1,10 do
		io.write('\n\tEarliest start time for activity '..act..': '..MySchedule:get_start(act))
	end
end

function create_forecasting_accuracy_table1_5()
	io.write('\nTable 1.5 of Measuring Time is now available in a text file')
	-- Create an output file with headings
	output_file=string.format('../tracking_table_1_5.txt')
	file=io.open(output_file, "w")
	file:write('\t\tEAC(t)\t\t\tEAC(t)\t\t\tEAC(t)\n')
	file:write('\tPV1\tPV2\tPV3\tED1\tED2\tED3\tES1\tES2\tES3\n')

	-- Calculate the forecast accuracy per period
	average_accuracy={0,0,0,0,0,0,0,0,0}

	-- Get the number of tracking periods and other data
	number_of_tracking_periods=MyTracking:get_number_of_tracking_periods()	-- 18 in this example
	nrmethods=9																-- 9 forecasting methods
	project_real_duration=number_of_tracking_periods						-- Since each tracking period has time length of 1 (cf. timePtr=1 in "simulate_progress_manually")

	-- Calculate the forecasting accuracy per period for the 9 forecasting methods
	for period=1,number_of_tracking_periods do
		file:write('\n',period)
		-- Get all EAC(t) results (9 forecasting methods)
		eact_vector={MyTracking:get_eac_time(period)}

		-- Calculate accuracy for the 9 time forecasting methods	
		for i,estimate in ipairs(eact_vector) do		
			forecast_accuracy=math.abs(project_real_duration-estimate)/project_real_duration*100
			file:write(string.format('\t%2.2f',forecast_accuracy))
			average_accuracy[i]=forecast_accuracy+average_accuracy[i]
		end
	end

	file:write('\nAvg.')
	for i=1,nrmethods do
		average=average_accuracy[i]/number_of_tracking_periods
		file:write(string.format('\t%2.2f',average))
	end
	
	-- Close the file
	file:flush()
	file:close()
end

function write_header(period_length,evm_id)

	if evm_id==1 then file:write('PV') 
	elseif evm_id==2 then file:write('AC')
	elseif evm_id==3 then file:write('EV') 
	end
	for t=1,period_length do
		file:write('\t',t)
	end
	file:write('\n')
end

function create_EVM_table1_6()
	io.write('\nTable 6.1 of Measuring Time is now available in a text file')
	-- Create a table with the EVM key metrics and performance measures and write to file
	-- (see table 1.6 of the book "Measuring Time")

	-- Create an empty file
	output_file=string.format('../tracking_table_1_6.txt')
	file=io.open(output_file,"w")

	-- Get the number of activities, the planned project duration and the number of tracking periods (which is equal to the real duration since each tracking period increases with 1 time unit in this example)
	nract=MySchedule:get_number_of_activities()					-- 10 in this example
	nrbaselineper=MySchedule:get_project_pd()					-- 16 in this example
	nrtrackper=MyTracking:get_number_of_tracking_periods()		-- 18 in this example

	-- Write the Planned Value (PV) for each activity for each period
	write_header(nrbaselineper,1)
	for act=1,nract do
		file:write(act,'\t')
		for period=1,nrbaselineper do
			file:write(MyTracking:get_activity_planned_value(act,period),'\t')
		end
		file:write('-\t-\n')
	end
	file:write('Total\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_pv(period),'\t')
	end
	file:write('\n\n')

	-- Write the Actual Cost (AC) for each activity for each period
	write_header(nrtrackper,2)
	for act = 1,nract do
		file:write(act,'\t')
		for period=1,nrtrackper do
			file:write(string.format('%2.2f\t',MyTracking:get_actual_cost(act,period)))
		end
		file:write('\n')
	end
	file:write('Total\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_ac(period)))
	end
	file:write('\n\n')

	-- Write the Earned Value (EV) for each activity for each period
	write_header(nrtrackper,3)
	for act = 1,nract do
		file:write(act,'\t')
		for period=1,nrtrackper do
			file:write(string.format('%2.2f\t',MyTracking:get_activity_earned_value(act,period)))
		end
		file:write('\n')
	end
	file:write('Total\t')
	for period=1,nrtrackper do
			file:write(string.format('%2.2f\t',MyTracking:get_project_ev(period)))
	end
	file:write('\n\n')

	-- Write the EVM performance measures
	write_header(nrtrackper,0)
	file:write('SV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_sv(period)))
	end
	file:write('\n')
	file:write('CV\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_cv(period)))
	end	
	file:write('\n')
	file:write('SPI\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spi(period)))
	end
	file:write('\n')
	file:write('CPI\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_cpi(period)))
	end
	file:write('\n\n')
	file:write('ES\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_es(period)))
	end
	file:write('\n')
	file:write('SV(t)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_svt(period)))
	end
	file:write('\n')
	file:write('SPI(t)\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spit(period)))
	end
	file:write('\n')
	
	-- Close the file
	file:flush()
	file:close()
end

main_procedure_chapter1()
