--[[
	Target: In this example script, it is shown how the project example published in the book 
			"Measuring Time - Improving Project Performance using Earned Value Management" 
			(Vanhoucke, page 34, table 2.2) can be reproduced using P2 Engine. 
	More information: www.p2engine.com
	Start with: main_procedure_chapter2
	Uses the following subprocedure(s): "subprocedure_create_objects", "subprocedure_read_files", and "subprocedure_manual_simulation"
	Classes: P2Project, P2Schedule, P2Reader and P2Tracking
	Last update: August 27, 2012
--]]

-- Define your path where you unzip your downloaded file from www.p2engine.com (MUST BE CHANGED BY THE USER!!)
MY_PATH=string.format('/Users/mariovanhoucke/Documenten van Mario/Lua Scripts/tutorial/')

-- Other paths are defined automatically (no need to change these paths!)
MY_SCRIPTS=MY_PATH..'MeasuringTimeExamples/'
MY_GENERAL=MY_PATH..'general/'
dofile(MY_SCRIPTS..'subprocedure_manual_simulation.lua')
dofile(MY_GENERAL..'subprocedure_read_files.lua')
dofile(MY_GENERAL..'subprocedure_create_objects.lua')

function main_procedure_chapter2()
	-- Create the three objects
	create_project_object()
	create_schedule_object()
	create_tracking_object()

	-- Read the project network of chapter 2 from a file 
	read_a_patterson_file(MY_SCRIPTS..'project_example_measuring_time_figure1_7.rcp')
	add_costs()

	-- Imitate project progress (manually)
	local RealStartTimes={0,0,0,2,1,6,10,5,12,9}
	local RealDurations={4,12,1,4,5,4,7,8,3,3}
	simulate_progress_manually(RealStartTimes,RealDurations)

	-- Create EVM data (table 2.2 of the book "Measuring Time")
	create_EVM_table2_2()		-- table 2.2

	delete_tracking_object()
	delete_schedule_object()
	delete_project_object()
	release_all_memory()
end

function add_costs()
	-- Assign costs per time unit to activities (cost per unit = total cost / activity duration)
	-- In the example, the costs per unit are assigned in a for loop using an array of costs
	cost_per_time_unit={20,4,25,20,4,120,3,1,2,20}
	for act,cost in ipairs(cost_per_time_unit) do
		MySchedule:set_variable_cost(act,cost)
	end
end

function create_EVM_table2_2()
	io.write('\nTable 2.2 of Measuring Time is now available in a text file')
	-- Create a table with the EVM key metrics and performance measures (including p-factor) and write to file
	-- (see table 2.2 of the book "Measuring Time")

	-- Create an empty file
	output_file=string.format('../tracking_table_2_2.txt')
	file=io.open(output_file,"w")

	-- Get the number of activities, the planned project duration and the number of tracking periods (which is equal to the real duration since each tracking period increases with 1 time unit in this example)
	nract=MySchedule:get_number_of_activities()					-- 10 in this example
	nrbaselineper=MySchedule:get_project_pd()					-- 16 in this example
	nrtrackper=MyTracking:get_number_of_tracking_periods()		-- 17 in this example

	-- Write header
	file:write('\t')
	for period=1,nrtrackper do
		file:write('\t',period)
	end

	file:write('\n\t\t\t\t\t\t\t\tTraditional Earned Value Calculations')

	-- Write the Earned Value (EV)
	file:write('\nEV\t\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_ev(period),'\t')
	end

	-- Write the Actual Cost (AC)
	file:write('\nAC\t\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_ac(period),'\t')
	end

	-- Write the Planned Value (PV)
	file:write('\nPV\t\t')
	for period=1,nrtrackper do
		file:write(MyTracking:get_project_pv(period),'\t')
	end

	file:write('\n')

	-- Write the EVM performance measures
	file:write('\nSV\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_sv(period)))
	end
	file:write('\nCV\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.0f\t',MyTracking:get_project_cv(period)))
	end	
	file:write('\nSPI\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spi(period)))
	end
	file:write('\nCPI\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_cpi(period)))
	end
	
	file:write('\n')
	file:write('\nES\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_es(period)))
	end
	file:write('\nSV(t)\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_svt(period)))
	end
	file:write('\nSPI(t)\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_spit(period)))
	end
	
	file:write('\n\t\t\t\t\t\t\t\tEffective Earned Value Calculations')
	file:write('\np-factor\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',MyTracking:get_project_p_factor(period)))
	end
	
	-- Effective EVM is not incorporated in P2 Engine but can be easily calculated as illustrated along the following lines
	-- Note: Try to avoid to program procedures yourself when available in P2 Engine

	-- Set the estimated portion of EV(r) that is usable and required no rework
	RPercent=0.9
		
	-- Calculate and write the effective Earned Value (EV(e))
	ev_e={}
	file:write('\nEV(e)\t\t')
	for period=1,nrtrackper do
		ev=MyTracking:get_project_ev(period)
		p=MyTracking:get_project_p_factor(period)
		ev_e[period]=ev*p+RPercent*(1-p)*ev
		file:write(string.format('%2.2f\t',ev_e[period]))
	end

	-- Write the EVM performance measures
	file:write('\nSV(e)\t\t')
	for period=1,nrtrackper do
		sv_e=ev_e[period]-MyTracking:get_project_pv(period)
		file:write(string.format('%2.2f\t',sv_e))
	end
	file:write('\nCV(e)\t\t')
	for period=1,nrtrackper do
		cv_e=ev_e[period]-MyTracking:get_project_ac(period)
		file:write(string.format('%2.2f\t',cv_e))
	end	
	file:write('\nSPI(e)\t\t')
	for period=1,nrtrackper do
		spi_e=ev_e[period]/MyTracking:get_project_pv(period)
		file:write(string.format('%2.2f\t',spi_e))
	end
	file:write('\nCPI(e)\t\t')
	for period=1,nrtrackper do
		cpi_e=ev_e[period]/MyTracking:get_project_ac(period)
		file:write(string.format('%2.2f\t',cpi_e))
	end

	-- Calculate and write the effective Earned Schedule (ES(e))
	es_e={}
	for period=1,nrtrackper do
		timePtr=0
		for t=1,nrtrackper do
			pvt=MyTracking:get_project_pv(t)
			if pvt>=ev_e[period] then 
				timePtr=t-1
				break
			end
		end	
		if timePtr==0 then
			pvt=0
		else
			pvt=MyTracking:get_project_pv(timePtr)
		end	
		pvtplus=MyTracking:get_project_pv(timePtr+1)
		es_e[period]=timePtr+(ev_e[period]-pvt)/(pvtplus-pvt)
	end

	file:write('\n\nES(e)\t\t')
	for period=1,nrtrackper do
		file:write(string.format('%2.2f\t',es_e[period]))
	end

	file:write('\nSV(t)(e)\t')
	for period=1,nrtrackper do
		sv_e=es_e[period]-MyTracking:get_status_date(period)
		file:write(string.format('%2.2f\t',sv_e))
	end

	file:write('\nSPI(t)(e)\t')
	for period=1,nrtrackper do
		spi_e=es_e[period]/MyTracking:get_status_date(period)
		file:write(string.format('%2.2f\t',spi_e))
	end
	
	-- Close the file
	file:flush()
	file:close()
end

main_procedure_chapter2()
