--[[
	Target: In this example script, table 4.3 of the simulation experiments as 
			published in the book "Measuring Time - Improving Project Performance using 
			Earned Value Management" (Vanhoucke, page 60) can be calculated using P2 Engine. 
			Note1: Values can differ due to settings of the projects and random numbers
			Note2: A similar script can be made for table 4.4 (replace "get_time_mape" by "get_time_mpe")
	More information: www.p2engine.com
	Start with: main_procedure_chapter4_part2
	Uses the following subprocedure(s): "subprocedure_create_objects" and "subprocedure_read_files"
	Classes: P2Project, P2Schedule, P2Reader and P2Simulator
	Last update: August 27, 2012
--]]

-- Define your path where you unzip your downloaded file from www.p2engine.com (MUST BE CHANGED BY THE USER!!)
MY_PATH=string.format('/Users/mariovanhoucke/Documenten van Mario/Lua Scripts/tutorial/')

-- Other paths are defined automatically (no need to change these paths!)
MY_SCRIPTS=MY_PATH..'MeasuringTimeExamples/'
MY_GENERAL=MY_PATH..'general/'
dofile(MY_GENERAL..'subprocedure_read_files.lua')
dofile(MY_GENERAL..'subprocedure_create_objects.lua')
file_directory=MY_PATH..'DataSetEvms_Patterson/Set 1 SP/'

function main_procedure_chapter4_part2()
	-- Create the objects
	create_project_object()
	create_schedule_object()
	create_simulator_object()
	create_reader_object()

	-- Parameters
	nrruns=100			-- Number of runs per simulation
	nrscenarios=9		-- Number of forecasting scenarios (table 4.2 of the book "Measuring Time")
	nrmethods=9			-- Number of forecasting methods (forecasting time)
	nrfiles=900			-- Number of files in the database
	mape_vector={}		-- mape_vector[fileID][methodID] with fileID = 1, .., nrfiles and methodID = 1, .., nrmethods
	open_output_file('accuracy_table_4_3.txt')
	
	-- Run the simulations
	for scenario=1,nrscenarios do	
		for teller=1,nrfiles do
			-- Reset the schedule object to delete the data of the previous run
			MySchedule:reset_schedule_object()

			-- Read the input files (set1 from Measuring Time, available at www.or-as.be/measuringtime)
			filename=file_directory..'EV'..teller..'.rcp'
			read_a_patterson_file(filename)
			add_random_costs(10,500)
			--io.write("\nNumber of activities = ",MySchedule:get_number_of_activities())
		
			-- Run the simulations
			simulate_9_scenarios(scenario,nrruns)
			-- Evaluate forecasting accuracy and save in memory
			evaluate_forecasting_accuracy(teller)
		end
		-- Create output files
		create_forecasting_accuracy_table4_3(scenario)
	end
	close_output_file()

	delete_reader_object()
	delete_simulator_object()
	delete_schedule_object()
	delete_project_object()
	release_all_memory()
end

function evaluate_forecasting_accuracy(fileID)
	-- Calculate the forecasting accuracy per period for the 9 forecasting methods
	mape_vector[fileID]={MySimulator:get_time_mape()}

	-- Write to screen (turn off to speed up process)
	io.write('\nFile ',fileID,'\t')
	for method=1,nrmethods do
		io.write(string.format('\t%2.2f',mape_vector[fileID][method]))
	end

end

function create_forecasting_accuracy_table4_3(scenarioID)
--	io.write('\nTable 4.3 of Measuring Time is now available in a text file')
	avg_mape={}
	for method=1,nrmethods do
		avg_mape[method]=0
		for teller=1,nrfiles do
			avg_mape[method]=avg_mape[method]+mape_vector[teller][method]
		end
	end
	file:write('\n',scenarioID,'\t')
	for method=1,nrmethods do
		-- Get all EAC(t) results (9 forecasting methods)
		file:write(string.format('\t%2.2f',100*avg_mape[method]/nrfiles))
	end
end

function open_output_file(filename)
	-- Create an output file with headings
	output_file=string.format('../'..filename)
	file=io.open(output_file, "w")
	file:write('\n\t\tEAC(t)\t\t\tEAC(t)\t\t\tEAC(t)')
	file:write('\nScenario\tPV1\tPV2\tPV3\tED1\tED2\tED3\tES1\tES2\tES3')
end

function close_output_file()
	-- Close the file
	file:flush()
	file:close()
end

function add_random_costs(lower,upper)
	for act=1,MySchedule:get_number_of_activities() do
		ran=math.random(lower,upper)
		MySchedule:set_variable_cost(act,ran)
	end
end

function simulate_9_scenarios(scenarioID,nrruns)
	-- reset_simulation_object: connect to new schedule (no need to make a new P2Simulator object)
	MySimulator:reset_simulation_object(MySchedule)

	-- Set time unit (= length of each period during project progress simulation)
	time_unit=1
	MySimulator:set_periodic_time_unit(time_unit)
	--MySimulator:set_callback("sim_callback")

--	io.write("\n Simulation run : \t",runID)
	MySimulator:set_callback("sim_callback")
	MySimulator:run_simulation(nrruns,scenarioID,1,0,0,0)
end

function sim_callback(p1,p2,p3)
--[[
	Procedure set in "set_callback"
	This procedure is called at every run (1 to nrruns) of each simulation and is a powerful tool
	to interrupt simulations based on conditions or to write data to a file for every simulation.
	This procedure is not used in this script and is therefore empty. 
	For more information:
	visit http://www.p2engine.com/p2simulator/set_callback 
	or look at other example scripts available at http://www.p2engine.com/examples
--]]
	return 0
end

main_procedure_chapter4_part2()
